#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#

"""OpenDataLab python SDK."""

from opendatalab.__version__ import __version__
from opendatalab.__version__ import __url__
from opendatalab.client.client import Client

__all__ = ["__url__", "__version__", "Client"]


#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
import logging
import os
import sys
import threading
import time
from concurrent.futures import ThreadPoolExecutor, as_completed
from pathlib import Path
from typing import List

import click
import oss2
from tqdm import tqdm

from opendatalab.cli.policy import service_agreement_url, private_policy_url
from opendatalab.cli.utility import ContextInfo, exception_handler
from opendatalab.exception import OdlDataNotExistsError

oss2.set_stream_logger(level=logging.CRITICAL)
key_to_get_size_map = {}


def handler(dwCtrlType):
    if dwCtrlType == 0:  # CTRL_C_EVENT
        pid = os.getpid()
        os.kill(pid, 9)


if sys.platform == "win32":
    import win32api
    win32api.SetConsoleCtrlHandler(handler, True)


def get_oss_traffic_limit(limit_speed):
    if limit_speed <= 0:
        return 0
    if limit_speed < 245760:
        return 245760
    if limit_speed > 838860800:
        return 838860800
    return limit_speed


def download_object(
        bucket: oss2.Bucket,
        obj_key: str,
        lock: threading.RLock,
        root: str,
        pbar: tqdm,
        limit_speed: int,
):
    def progress_callback(bytes_consumed, _):
        with lock:
            global key_to_get_size_map
            if obj_key not in key_to_get_size_map:
                key_to_get_size_map[obj_key] = 0

            # sys.stdout.flush()
            pbar.update(bytes_consumed - key_to_get_size_map[obj_key])
            key_to_get_size_map[obj_key] = bytes_consumed

    try:
        headers = dict()
        if limit_speed > 0:
            headers[oss2.models.OSS_TRAFFIC_LIMIT] = str(limit_speed)

        filename = os.path.join(root, obj_key.split("/")[-1])

        oss2.resumable_download(
            bucket,
            obj_key,
            filename,
            multiget_threshold=50 * 1024 * 1024,  # 50M -> 500G(cdn)
            part_size=10 * 1024 * 1024,  # 10M
            progress_callback=progress_callback,
            num_threads=1,
            headers=headers,
        )
        return True, None
    except oss2.exceptions.InconsistentError as e:
        return False, e
    except oss2.exceptions.ServerError as e:
        return False, e
    except Exception as e:
        return False, e


@exception_handler
def implement_get(obj: ContextInfo, name: str, thread: int, limit_speed: int, compressed: bool = True) -> None:
    """
    implementation for getting dataset files
    Args:
        obj (ContextInfo):
        name (str):
        thread (int):
        limit_speed (int):
        compressed (bool):
    Returns:
    """
    ds_split = name.split("/")
    if len(ds_split) > 1:
        dataset_name = ds_split[0]
        sub_dir = "/".join(ds_split[1:])
    else:
        dataset_name = name
        sub_dir = ""

    client = obj.get_client()
    info_data_name = client.get_api().get_info(dataset_name)['name']
    dataset = client.get_dataset(info_data_name)
    prefix = dataset.get_object_key_prefix(compressed)
    bucket = dataset.get_oss_bucket()

    total_files, total_size = 0, 0
    obj_info_list = []
    download_info_body = []

    for info in oss2.ObjectIteratorV2(bucket, prefix):
        if not info.is_prefix() and not info.key.endswith("/"):
            file_name = "/".join(info.key.split("/")[2:])
            f_name = Path(file_name).name
            if not sub_dir:
                obj_info_list.append(info.key)
                total_files = total_files + 1
                total_size = total_size + info.size
                download_info_body.append({"name": f_name, "size": info.size})
            elif sub_dir and file_name.startswith(sub_dir):
                obj_info_list.append(info.key)
                total_files = total_files + 1
                total_size = total_size + info.size
                download_info_body.append({"name": f_name, "size": info.size})
            else:
                pass

    if len(download_info_body) == 0:
        raise OdlDataNotExistsError(error_msg=f"{name} not exists!")

    client.get_api().call_download_log(dataset_name, download_info_body)
    click.echo(f"Scan done, total files: {len(obj_info_list)}, total size: {tqdm.format_sizeof(total_size,divisor=1024)}")

    download_data = client.get_api().get_download_record(dataset_name)
    has_download = download_data['hasDownload']

    if not has_download:
        if click.confirm(f"<<User Service Agreement>>: {service_agreement_url}"
                         f"\n<<Privacy Policy>>: {private_policy_url}"
                         f"\n[Warning]: Before downloading, please agree above content."):
            client.get_api().submit_download_record(dataset_name, download_data)
        else:
            click.secho('bye~')
            sys.exit(1)

    limit_speed_per_thread = get_oss_traffic_limit(int(limit_speed * 1024 * 8 / thread))

    local_dir = Path.cwd().joinpath(dataset_name)
    if click.confirm(f"Download files into local directory: {local_dir} ?", default=True):
        if not Path(local_dir).exists():
            Path(local_dir).mkdir(parents=True)
            print(f"create local dir: {local_dir}")
    else:
        click.secho('bye~')
        sys.exit(1)

    pbar = tqdm(total=total_size, unit="B", unit_divisor=1024, unit_scale=True, position=0)

    index = 0
    is_running = True
    while is_running:
        global key_to_get_size_map
        bucket = dataset.refresh_oss_bucket()
        error_object_list = get_objects_retry(bucket=bucket,
                                              local_dir=local_dir,
                                              obj_info_list=obj_info_list,
                                              pbar=pbar,
                                              limit_speed_per_thread=limit_speed_per_thread,
                                              thread=thread)
        index = index + 1
        time.sleep(1)
        if len(error_object_list) > 0:
            obj_info_list = error_object_list
            is_running = True
            continue
        else:
            is_running = False
            break

    pbar.close()
    print(f"{dataset_name} ,download completed!")


def get_objects_retry(bucket, local_dir, obj_info_list, pbar, limit_speed_per_thread, thread) -> List:
    lock = threading.RLock()
    error_object_list = []
    with ThreadPoolExecutor(max_workers=thread) as executor:
        future_to_obj = {executor.submit(
            download_object, bucket, obj, lock, local_dir, pbar, limit_speed_per_thread
        ): obj for obj in obj_info_list}

        for future in as_completed(future_to_obj):
            obj = future_to_obj[future]
            success, _ = future.result()
            if not success:
                error_object_list.append(obj)

    return error_object_list



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#

"""Client Module"""

from opendatalab.client.client import Client
from opendatalab.cli.config import config

__all__ = [
    "Client",
    "config",
]


#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#

"""OpenDataLab CLI utility functions."""
import sys
import json
from functools import wraps
from typing import Any, Callable, TypeVar
import click
from opendatalab.__version__ import __version__

from opendatalab.cli.config import config as client_config
from opendatalab.client import Client
from opendatalab.utils import UUID
from opendatalab.exception import OpenDataLabError

_Callable = TypeVar("_Callable", bound=Callable[..., None])


class ContextInfo:
    """This class contains command context."""

    def __init__(self, url: str, token: str):
        self.url = url
        self.token = token
        self.config = client_config
        self.conf_file = client_config._get_config_filepath()
        self._conf_content = self.check_config()
        odl_cookie = self._conf_content['user.token'] if self._conf_content['user.token'] else ""
        self.cookie = odl_cookie

        self.check_ret = 0
        self.install_version = __version__
        self.latest_version = None

    def get_client(self) -> Client:
        return Client(self.url, self.token, self.cookie)

    def get_content(self):
        return self._conf_content

    def set_content(self, content: dict) -> None:
        for key, value in content.items():
            self._conf_content[key] = value

            if key == 'user.token' and not content[key]:
                self.cookie = content[key]

    def get_config_content(self):
        try:
            with open(self.conf_file, 'r') as f:
                config_content = json.load(f)
        except json.decoder.JSONDecodeError:
            config_content = {}

        return config_content

    def check_config(self):
        res = self.get_config_content()
        if not res:
            init_config_dict = {
                'endpoint': self.url,
                'user.email': '',
                'user.token': '',
                'odl_anonymous': UUID,
            }
            result = init_config_dict
            with open(self.conf_file, 'w') as f:
                json.dump(init_config_dict, f, indent=4, sort_keys=True, separators=(',', ':'))
        else:
            result = res

        return result

    def update_config(self, content: dict) -> None:
        res = self.get_config_content()
        if res:
            self.set_content(content)
            with open(self.conf_file, 'w') as f:
                f.seek(0)
                json.dump(self._conf_content, f, indent=4, sort_keys=True, separators=(',', ':'))

    def clean_config(self):
        res = self.get_config_content()
        if not res:
            self.check_config()
        else:
            with open(self.conf_file, "w") as f:
                if 'user.token' in res.keys():
                    res['user.token'] = ''
                if 'user.email' in res.keys():
                    res['user.email'] = ''
                f.seek(0)
                json.dump(res, f, indent=4, sort_keys=True, separators=(',', ':'))

        return res

    def set_check_info(self, latest_version, check_ret):
        self.latest_version = latest_version
        self.check_ret = check_ret

    def get_check_ret(self):
        return self.check_ret


def implement_cli(ctx: click.Context, url: str, token: str) -> None:
    ctx.obj = ContextInfo(url, token)
    # ctx.obj.check_config()


def error(message: str):
    """Print the error message and exit the program.

    Arguments:
        message: The error message to echo.

    """
    click.secho(f"ERROR: {message}", err=True, fg="red")
    sys.exit(1)


def exception_handler(func: _Callable) -> _Callable:
    """Decorator for CLI functions to catch custom exceptions.

    Arguments:
        func: The CLI function needs to be decorated.

    Returns:
        The CLI function with exception catching procedure.

    """

    @wraps(func)
    def wrapper(*args: Any, **kwargs: Any) -> Any:
        try:
            func(*args, **kwargs)
        except OpenDataLabError as err:
            if err.STATUS_CODE == 401:
                click.secho(f"Error: authentication failure, please login!", err=True, fg='red')
                pass
            elif err.STATUS_CODE == 403:
                click.secho(f"Unable to access. Please visit the dataset homepage!", err=True, fg='red')
                pass
            elif err.STATUS_CODE == 404:
                click.secho(f"Data not exists!", err=True, fg='red')
                pass
            elif err.STATUS_CODE == 412:
                click.secho(f"Access with cdn error!", err=True, fg='red')
                pass
            elif err.STATUS_CODE == 500:
                click.secho(f"Internal server occurs!", err=True, fg='red')
                pass
            else:
                click.secho(f"Error occurs!!!", err=True, fg='red')

            sys.exit(1)

    return wrapper  # type: ignore[return-value]



#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
import http
import sys

import click
import oss2
import requests
from requests.adapters import HTTPAdapter

from opendatalab.client.api import OpenDataLabAPI
from opendatalab.exception import OpenDataLabError
from opendatalab.utils import parse_url, get_api_token_from_env


class Dataset:
    def __init__(self, url: str, token: str = "", odl_cookie: str = "") -> None:
        host, dataset_name = parse_url(url)
        self.dataset_name = dataset_name
        if token == "":
            token = get_api_token_from_env()
        self.open_data_lab_api = OpenDataLabAPI(host, token, odl_cookie)

        self.oss_bucket = None
        self.bucket_name = None
        self.oss_path_prefix = ""
        self.init_oss_bucket()

    def get(self, filepath: str, compressed: bool = True):
        object_key = self.get_object_key_prefix(compressed) + filepath
        try:
            return self.oss_bucket.get_object(object_key)
        except oss2.exceptions.ServerError as e:
            if "InvalidAccessKeyId" not in str(e):
                raise e

            self.init_oss_bucket()
            return self.oss_bucket.get_object(object_key)

    def init_oss_bucket(self, expires=3600):
        sts = self.open_data_lab_api.get_dataset_sts(self.dataset_name, expires=expires)

        if sts:
            path_info = sts["path"].replace("oss://", "").split("/")
            bucket_name = path_info[0]
            sts_point, sts_use_cname = self.select_endpoint(sts)

            if sts_point:
                auth = oss2.StsAuth(sts["accessKeyId"], sts["accessKeySecret"], sts["securityToken"])
                self.oss_bucket = oss2.Bucket(auth, sts_point, bucket_name, is_cname=sts_use_cname)
                self.oss_path_prefix = "/".join(path_info[1:])
            else:
                raise OpenDataLabError(1001, "access to bucket error")

    def get_oss_bucket(self) -> oss2.Bucket:
        if self.oss_bucket is None:
            self.init_oss_bucket()
        return self.oss_bucket

    def refresh_oss_bucket(self) -> oss2.Bucket:
        self.init_oss_bucket()
        return self.get_oss_bucket()

    def get_object_key_prefix(self, compressed: bool = True) -> str:
        if compressed:
            return f"{self.oss_path_prefix}/raw/"
        else:
            return f"{self.oss_path_prefix}/"

    @classmethod
    def select_endpoint(cls, sts):
        s = requests.Session()
        sts_endpoints = sts["endpoints"]
        path_info = sts["path"].replace("oss://", "").split("/")
        bucket_name = path_info[0]

        # use general endpoint
        if len(sts_endpoints) > 0:
            endpoint = sts_endpoints[-1]
            sts_endpoint = endpoint['url']
            sts_use_cname = endpoint['useCname']

            url_splitter = "://"
            url_split_arr = str(sts_endpoint).split(url_splitter)
            url_prefix = url_split_arr[0]
            url_body = url_split_arr[1]
            check_url = url_prefix + url_splitter + bucket_name + "." + url_body + "/check_connected"
            s.mount(check_url, HTTPAdapter(max_retries=0))

            try:
                resp = s.get(check_url, timeout=(3, 1)) # 0.5
                if resp.status_code == http.HTTPStatus.OK:
                    return sts_endpoint, sts_use_cname
            except Exception as e:
                click.secho(f"ConnectionError occurs, please check network!", fg='red')
                sys.exit(-1)


#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
import sys

import oss2
from rich import box
from rich.console import Console
from rich.table import Table

from opendatalab.cli.utility import ContextInfo, exception_handler
from opendatalab.exception import OdlAccessDeniedError
from opendatalab.utils import bytes2human


@exception_handler
def implement_ls(obj: ContextInfo, dataset: str) -> None:
    """
    implementation for show dataset files
    Args:
        obj (ContextInfo):
        dataset (str): dataset name

    Returns:

    """
    ds_split = dataset.split("/")
    if len(ds_split) > 1:
        dataset_name = ds_split[0]
        sub_dir = "/".join(ds_split[1:])
    else:
        dataset_name = dataset
        sub_dir = ""

    client = obj.get_client()
    info_data_name = client.get_api().get_info(dataset_name)['name']
    dataset_instance = client.get_dataset(dataset_name=info_data_name)

    bucket = dataset_instance.get_oss_bucket()
    prefix = dataset_instance.get_object_key_prefix(compressed=True)

    object_info_dict = {}
    total_files, total_size = 0, 0
    for info in oss2.ObjectIteratorV2(bucket, prefix):
        if not info.is_prefix() and not info.key.endswith("/"):
            file_name = "/".join(info.key.split("/")[2:])
            if not sub_dir:
                object_info_dict[file_name] = bytes2human(info.size)
                total_files = total_files + 1
                total_size = total_size + info.size
            elif sub_dir and file_name.startswith(sub_dir):
                object_info_dict[file_name] = bytes2human(info.size)
                total_files = total_files + 1
                total_size = total_size + info.size
            else:
                pass

    if len(object_info_dict) == 0:
        raise OdlAccessDeniedError()
        sys.exit(-1)

    sorted_object_info_dict = dict(sorted(object_info_dict.items(), key=lambda x: x[0], reverse=True))

    console = Console()
    table = Table(show_header=True, header_style='bold cyan', box=box.ASCII2)  # ROUNDED
    table.add_column("File Name", min_width=20, justify='left')
    table.add_column("Size", width=12, justify='left')

    print(f"total: {total_files}, size: {bytes2human(total_size)}")
    for key, val in sorted_object_info_dict.items():
        table.add_row(key, val, end_section=True)

    console.print(table)



import sys

import click
import requests
import json
import time
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5
from base64 import b64encode, b64decode

from opendatalab.__version__ import uaa_url_prefix, odl_clientId

api_login = "/api/v1/login/byClientSdk"
api_public_key = "/api/v1/cipher/getPubKey"
api_user_info = "/api/v1/login/getUserInfo"
api_auth = "/api/v1/internal/auth"


clientId = odl_clientId
public_key_url = uaa_url_prefix + api_public_key
login_url = uaa_url_prefix + api_login
user_info_url = uaa_url_prefix + api_user_info
auth_url = uaa_url_prefix + api_auth


def get_public_key():
    public_param = {"from": "browser", "type": "login", "clientId": clientId}  # "platform" "browser"
    data = json.dumps(public_param)
    resp = requests.post(url=public_key_url,
                         data=data,
                         headers={"Content-Type": "application/json"}
                         )

    result = ""
    if resp.status_code == 200:
        result = resp.json()['data']['pubKey']

    return result


def rsa_encrypt(content):
    public_key = get_public_key()
    key = b64decode(public_key)
    key = RSA.importKey(key)

    cipher = PKCS1_v1_5.new(key)
    cipher_text = b64encode(cipher.encrypt(bytes(content, 'utf-8')))

    return cipher_text


def get_account(account, password):
    timestamp_str = str(time.time()).split('.')[0]
    raw_text = f"{clientId}||{password}||{timestamp_str}"

    encrypt_key = rsa_encrypt(raw_text)
    key_str = encrypt_key.decode('utf-8')

    user_info = {'account': account, 'password': key_str, 'autoLogin': 'true'}
    data = json.dumps(user_info)

    resp = requests.post(url=login_url,
                         data=data,
                         headers={"Content-Type": "application/json"}
                         )

    result = None
    authorization = None
    sso_uid = None
    if resp.status_code == 200:
        result = resp.json()['data']

    if result:
        authorization = resp.headers['authorization']
        sso_uid = resp.json()['data']['ssoUid']
    else:
        click.secho(f"Error: Auth failure with account: {account}", err=True, fg="red")
        sys.exit(1)

    return authorization, sso_uid


def get_user_info(authorization):
    result = None
    if authorization:
        resp = requests.post(url=user_info_url,
                             headers={
                                 "Authorization": f"{authorization}",
                             }
                             )

        if resp.status_code == 200:
            result = resp.json()['data']['ssoUid']

    return result


def get_auth_code(sso_uid):
    result = None
    if sso_uid:
        client_id = {'clientId': clientId}
        data = json.dumps(client_id)

        resp = requests.post(url=auth_url,
                             data=data,
                             headers={
                                 "Content-Type": "application/json",
                                 "Cookie": f"ssouid={sso_uid}",
                             }
                             )
        if resp.status_code == 200:
            result = resp.json()['data']['code']

    return result


def get_odl_token(account, password):
    authorization, sso_uid = get_account(account=account, password=password)
    auth_code = None
    if sso_uid:
        auth_code = get_auth_code(sso_uid=sso_uid)

    if not auth_code:
        click.secho(f"Error: Auth failure with account: {account}", err=True, fg="red")
        sys.exit(1)

    return auth_code



#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
import json
import sys

import click
import requests

from opendatalab.__version__ import __version__
from opendatalab.client.uaa import get_odl_token
from opendatalab.exception import *
from opendatalab.utils import UUID


class OpenDataLabAPI(object):
    def __init__(self, host, token, odl_cookie):
        self.host = host
        self.token = token
        self.odl_cookie = odl_cookie

    def get_dataset_sts(self, dataset, expires=900):
        """Get dataset sts by dataset_name
        Args:
            expires (int): expire timeout unit seconds
            dataset (string): dataset_name

        Raises:
            OpenDataLabError: 

        Returns:
            string: json data from response
        """
        data = {"expires": expires, }
        resp = requests.get(
            url=f"{self.host}/api/datasets/{dataset}/sts",
            params=data,
            headers={
                "X-OPENDATALAB-API-TOKEN": self.token,
                "Cookie": f"opendatalab_session={self.odl_cookie}",
                "User-Agent": UUID,
            }, )

        if resp.status_code != 200:
            if resp.status_code == 404:
                raise OdlDataNotExistsError()
            elif resp.status_code == 401:
                raise OdlAuthError()
            elif resp.status_code == 403:
                raise OdlAccessDeniedError()
            elif resp.status_code == 412:
                raise OdlAccessCdnError()
            elif resp.status_code == 500:
                raise OdlAccessDeniedError()
            else:
                raise RespError(resp_code=resp.status_code, error_msg=resp.reason)
        # print(f"sts api, headers: {resp.headers}, text: {resp.text}")
        return resp.json()["data"]

    def search_dataset(self, keywords):
        resp = requests.post(  # f"{self.host}/api/datasets/?pageSize=25&keywords={keywords}",
            f"{self.host}/api/datasets/list",
            headers={"X-OPENDATALAB-API-TOKEN": self.token,
                     "Cookie": f"opendatalab_session={self.odl_cookie}",
                     "User-Agent": f"opendatalab-python-sdk/{__version__}",
                     },
            data=json.dumps({
                "backend": False,
                "keywords": keywords,
                "pageSize": 25,
                "state": ["online"],
            })
        )
        if resp.status_code != 200:
            print(f"{OpenDataLabError(resp.status_code, resp.text)}")
            sys.exit(-1)

        result_list = resp.json()["data"]["list"]
        if not result_list:
            click.secho(f"No datasets matched!", fg='red')
            sys.exit(-1)

        return result_list

    def get_similar_dataset(self, dataset):
        dataset_id = int(self.get_info(dataset)['id'])
        resp = requests.get(
            f"{self.host}/api/datasets/{dataset_id}/similar",
            headers={"X-OPENDATALAB-API-TOKEN": self.token,
                     "Cookie": f"opendatalab_session={self.odl_cookie}",
                     "User-Agent": f"opendatalab-python-sdk/{__version__}",
                     },
        )
        if resp.status_code != 200:
            # print(f"{(resp.status_code, resp.text)}")
            sys.exit(-1)

        data = resp.json()['data']
        return data

    def get_info(self, dataset):
        resp = requests.get(
            f"{self.host}/api/datasets/{dataset}?backend=false",
            headers={"X-OPENDATALAB-API-TOKEN": self.token,
                     "Cookie": f"opendatalab_session={self.odl_cookie}",
                     "User-Agent": f"opendatalab-python-sdk/{__version__}",
                     },
        )
        if resp.status_code != 200:
            click.echo(f"dataset: {dataset}, get info failure, error_code: {resp.status_code}")
            sys.exit(-1)

        data = resp.json()["data"]
        if data['id'] == 0:
            click.secho(f"No dataset: {dataset}", fg='red')
            sys.exit(-1)

        return data

    def call_download_log(self, dataset, download_info):
        dataset_id = int(self.get_info(dataset)['id'])
        data = json.dumps(download_info)

        resp = requests.post(
            f"{self.host}/api/track/datasets/download/{dataset_id}",
            data=data,
            headers={"Content-Type": "application/json",
                     "Cookie": f"opendatalab_session={self.odl_cookie}",
                     "User-Agent": f"opendatalab-python-sdk/{__version__}",
                     },
        )

        if resp.status_code != 200:
            raise OpenDataLabError(resp.status_code, resp.text)

    def get_download_record(self, dataset):
        dataset_id = int(self.get_info(dataset)['id'])
        resp = requests.get(
            f"{self.host}/api/datasets/{dataset_id}/download",
            headers={"Content-Type": "application/json",
                     "Cookie": f"opendatalab_session={self.odl_cookie}",
                     "User-Agent": f"opendatalab-python-sdk/{__version__}",
                     },
        )

        if resp.status_code != 200:
            raise OpenDataLabError(resp_code=resp.status_code, error_msg=resp.reason)

        data = resp.json()["data"]
        return data

    def submit_download_record(self, dataset, download_data):
        dataset_id = int(self.get_info(dataset)['id'])

        profession = "OTHER" if not download_data['profession'] else download_data['profession']
        purpose = ["OTHER"] if not download_data['purpose'] else download_data['purpose']
        expand = ["OTHER"] if not download_data['expand'] else download_data['expand']
        data = {
            "expand": expand,
            "profession": profession,
            "purpose": purpose
        }
        data = json.dumps(data)

        resp = requests.put(
            f"{self.host}/api/datasets/{dataset_id}/download",
            data=data,
            headers={"Content-Type": "application/json",
                     "Cookie": f"opendatalab_session={self.odl_cookie}",
                     "User-Agent": f"opendatalab-python-sdk/{__version__}",
                     },
        )

        if resp.status_code != 200:
            raise OpenDataLabError(resp_code=resp.status_code, error_msg=resp.reason)

    def odl_auth(self, account, password):
        code = get_odl_token(account, password)
        data = {
            "code": code,
            "redirect": "",
        }
        data = json.dumps(data)

        resp = requests.post(
            f"{self.host}/api/users/auth",
            data=data,
            headers={"Content-Type": "application/json"},
        )

        if resp.status_code != 200:
            raise OdlAuthError(resp.status_code, resp.text)

        odl_token = resp.json()["data"]["token"]
        config_json = {
            'user.email': account,
            'user.token': odl_token,
        }

        return config_json

    def check_version(self):
        resp = requests.get(
            f"{self.host}/api/sdk/checkVersion",
            headers={"Content-Type": "application/json"},
        )

        if resp.status_code != 200:
            raise OdlAuthError(resp.status_code, error_msg=resp.text)
            sys.exit(-1)

        version_info = resp.json()["data"]
        return version_info



#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
# from datetime import date


"""OpenDataLab python SDK version info."""

__url__ = "http://opendatalab-test.shlab.tech" # "https://opendatalab.com"
__version__ = "0.0.3b02"
__svc__ = '1.9'
odl_clientId = "ypkl8bwo0eb5ao1b96no"  # "kmz3bkwzlaa3wrq8pvwa"
uaa_url_prefix = "http://uaa-staging.mai4u.com/gw/uaa-be"  # "https://sso.openxlab.org.cn/gw/uaa-be"



#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
from typing import Optional


class OpenDataLabError(Exception):
    """
    Base class for OpenDataLab custom exceptions.
    """

    def __init__(self, resp_code: Optional[int] = None, error_msg: str = ""):
        super().__init__(self, error_msg)
        self.resp_code = resp_code
        self.error_msg = error_msg

    def __str__(self) -> str:
        if self.resp_code:
            return f"{self.resp_code}: {self.error_msg}"
        else:
            return f"{self.error_msg}"


class RespError(OpenDataLabError):
    """

    """
    STATUS_CODE: int
    _INDENT = " " * len(__qualname__)  # type: ignore

    def __init__(self, resp_code: Optional[int] = None, error_msg: str = ""):
        super().__init__(resp_code, error_msg)

    def __init_subclass__(cls, **kwargs) -> None:
        cls._INDENT = " " * len(cls.__name__)

    def __str__(self) -> str:
        if hasattr(self, "resp_code"):
            return f"Error: {self.STATUS_CODE}, error_msg: {self.error_msg}"

        return super().__str__()


class OdlAuthError(RespError):
    STATUS_CODE = 401


class OdlAccessDeniedError(RespError):
    STATUS_CODE = 403


class OdlDataNotExistsError(RespError):
    STATUS_CODE = 404


class OdlAccessCdnError(RespError):
    STATUS_CODE = 412


class InternalServerError(RespError):
    STATUS_CODE = 500



#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
from opendatalab.client.api import OpenDataLabAPI
from opendatalab.dataset.dataset import Dataset
from opendatalab.utils import get_api_token_from_env
from opendatalab.__version__ import __url__


class Client:
    def __init__(self, host: str = __url__, token: str = "", odl_cookie: str = ""):
        """opendatalab client

        Args:
            host str: "https://opendatalab.com".
            token str: Defaults to "".
            odl_cookie str: Defaults to "".
        """
        self.host = host
        self.token = token
        self.odl_cookie = odl_cookie
        if self.token == "":
            self.token = get_api_token_from_env()
        self.dataset_map = {}
        self.odl_api = None

    def get_dataset(self, dataset_name: str) -> Dataset:
        if dataset_name not in self.dataset_map:
            self.dataset_map[dataset_name] = Dataset(
                f"{self.host}/datasets/{dataset_name}", self.token, self.odl_cookie)
        return self.dataset_map[dataset_name]

    def get(self, dataset_name: int, filepath: str):
        dataset = self.get_dataset(dataset_name)
        return dataset.get(filepath)

    def get_api(self):
        self.odl_api = OpenDataLabAPI(self.host, self.token, self.odl_cookie)
        return self.odl_api
